/*
 * Copyright 2001 - 2002 Alexander Kanavin. All rights reserved.
 */
package org.kanavin.alex.translator;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import java.util.*;
import java.io.*;
import javax.microedition.io.*;
import org.kanavin.alex.translator.About;
import org.kanavin.alex.translator.Languages;

/* This application provides a simple interface to online translation engine
 * Babelfish. You enter a word or a whole phrase, choose from and to language 
 * and get the tranlsation. The program also remembers last language pair 
 * you used (even though due to bugs in MIDP implementation it can't use it to
 * set default langugage menu selection. */

public class Translator extends MIDlet implements CommandListener {

    static String BABELURL=new String("http://babelfish.altavista.com/tr");

    private Display display;

    private Command nextCommand = new Command("Next",Command.OK,1);
    private Command newCommand = new Command("New",Command.SCREEN,1);
    private Command exitCommand = new Command("Exit",Command.SCREEN,2);
    private Command aboutCommand = new Command("About",Command.SCREEN,2);
    private Command backCommand = new Command("Back",Command.SCREEN,2);

    int commandType;
    Vector PastPhrases;
    Languages langs;

    List pastPhrasesMenu;
    TextBox phraseBox;
    List langChoice;
    int lastLang;
    Form result;

    RecordStore lastLangStore = null;
    RecordEnumeration re = null;

    /* In the constructor a last language number is loaded from a 
     * database. Also, all GUI elements are built. */
    public Translator() throws RecordStoreException,RecordStoreNotFoundException,RecordStoreFullException {

	lastLangStore = RecordStore.openRecordStore("lastlang",true);
	re = lastLangStore.enumerateRecords (null, null, true);
	try {
	    re.reset();
	    if (re.hasNextElement()) {
		int id = re.nextRecordId();
		ByteArrayInputStream bais = new ByteArrayInputStream(lastLangStore.getRecord(id));
		DataInputStream inputStream = new DataInputStream(bais);
		lastLang = inputStream.readInt();
	    } else {
	   	lastLang = 0;
	    }
	} catch (Exception e) {
      	    lastLang = 0;
	}

	phraseBox = new TextBox("Word or phrase to translate (English, French or German):","",2000,TextField.ANY);
	phraseBox.addCommand(nextCommand);
	phraseBox.addCommand(aboutCommand);
	phraseBox.addCommand(exitCommand);

	phraseBox.setCommandListener(this);
        langs = new Languages();
        langChoice = langlist();
	
	result = new Form("Result:");
	result.append("");
	result.append("means\n");
	result.append("");
	result.addCommand(newCommand);
	result.addCommand(backCommand);
	result.addCommand(aboutCommand);
	result.addCommand(exitCommand);
	result.setCommandListener(this);
    }

/* Constructs available translation options list */
    List langlist() {
	LanguagePair lang;
	Enumeration e;

	List l = new List("Select:",List.IMPLICIT);
	l.addCommand(backCommand);
	l.addCommand(aboutCommand);
	l.addCommand(exitCommand);
//	l.addCommand(nextCommand);
	for (e = langs.elements(); e.hasMoreElements();) {
	    lang = (LanguagePair)e.nextElement();
	    l.append (lang.name, null);
	}
	l.setSelectedIndex(lastLang,true);
	l.setCommandListener(this);
	return l;
    }

/* These are required midlet methods */
    public void startApp() {
	display = Display.getDisplay(this);

	display.setCurrent(phraseBox);

    }

    public void pauseApp() { }

    /* Don't forget to close the databases. */
    public void destroyApp(boolean unconditional) { 
	try { 
	    lastLangStore.closeRecordStore();
	} catch (Exception e) {};
	notifyDestroyed();
    }

    /* Actions on commands. */      
    public void commandAction(Command c, Displayable s) {
	if (s instanceof List) {
	    List menu = (List) s;
	    if (menu == pastPhrasesMenu) {
	    } else if (menu == langChoice) {
		if (c==exitCommand) {
		    destroyApp(false);
	    	} else if (c==aboutCommand) {
		    About.showAbout(display);
	    	} else if (c==backCommand) {
		    display.setCurrent(phraseBox);
		} else {
		    lastLang = langChoice.getSelectedIndex();
		    saveLang();
		    String source = phraseBox.getString();
		    String langpair = ((LanguagePair)langs.elementAt(lastLang)).id;
		    result.set(0,new StringItem(null,source+"\n"));
//		    result.set(2,new StringItem(null,langpair));
		    result.set(2,new StringItem(null,getBabel(source,langpair)));
		    display.setCurrent(result);
		}
	    }
	} else if (s instanceof Form) {
	    Form form = (Form) s;
	    if (c==exitCommand){
		destroyApp(false);
	    } else if (c==aboutCommand) {
		About.showAbout(display);
    	    } else if (c==backCommand) {
		langChoice.setSelectedIndex(lastLang,true);
	        display.setCurrent(langChoice);
	    } else if (c==newCommand) {
		phraseBox.setString("");
		display.setCurrent(phraseBox);
	    }
	} else if (s instanceof TextBox) {
	    TextBox box = (TextBox) s;
	    if (c==exitCommand){
		destroyApp(false);
	    } else if (c==aboutCommand) {
		About.showAbout(display);
	    } else { // if (c==nextCommand) {
		langChoice.setSelectedIndex(lastLang,true);
		display.setCurrent(langChoice);
	    } 
	    
	}
    }

    /* Get translated data from Babelfish website. Source is source phrase,
     * langpair is babelfish id (en_de etc). */
    String getBabel(String source,String langpair)
    {	
	HttpConnection c = null;
	InputStream is = null;
	OutputStream os = null;
	StringBuffer babeldata = new StringBuffer();
	int ch;
        byte[] data;
	
	try {
	    String url = BABELURL+"?urltext="+getEncString(source)+"&lp="+langpair;
	    c = (HttpConnection)Connector.open(url);


//	    c.setRequestMethod(HttpConnection.POST);
//	    c.setRequestProperty("Content-Type","application/x-www-form-urlencoded");

//	    os = c.openOutputStream();

//          os.write("doit=done&tt=urltext&urltext=speak&url=http%3A%2F%2f&lp=en_es".getBytes());
//          os.flush(); 
            is = c.openInputStream();
            long len = c.getLength();
            if (len > 0) {
                data = new byte[(int)len];
                is.read(data);
		for (int i = 0; i < data.length; i++) {
		    if (data[i] > 0) {
	  		babeldata.append((char) data[i]);
		    } else {
			babeldata.append((char)(((data[i]& 0x1F) << 6) | (data[i+1] & 0x3F)));
			i++;
		    }
		}
            } else {
		while ((ch = is.read()) != -1) {
		    if (ch > 0) {
    		        babeldata.append((char) ch);
		    } else {
			babeldata.append((char)(((ch& 0x1F) << 6) | (is.read() & 0x3F)));			
		    }
                }
            }
//	    os.close();
	    is.close();
	    c.close();
	} catch (Exception e) {}
	Babel babel = new Babel(babeldata.toString());
	return (babel.makeString());
    }

/* Replaces spaces in source with %20s */    
    String getEncString(String source)
    {
	int index1=0;
	int index2=0;
	StringBuffer out = new StringBuffer();
	
	source = source.trim();
	while ((index2=source.indexOf(" ",index1)) >= 0 ) {
	    out.append(source.substring(index1,index2)+"%20");
	    index1=index2+1;
	}
	out.append(source.substring(index1,source.length()));
	return out.toString();
    }

/* Saves last language pair used in a database to use it next time midlet is run */
    void saveLang()
    {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	DataOutputStream outputStream = new DataOutputStream(baos);
	try {
	    outputStream.writeInt(lastLang);
	} catch (Exception e) {}
	byte[] b = baos.toByteArray();
	try {
	    re.reset();
	    if (re.hasNextElement()) {
		int id = re.nextRecordId();
		lastLangStore.setRecord(id,b,0,b.length);
	    } else
		lastLangStore.addRecord(b, 0, b.length);
	} catch (Exception e) {}
    }

}

